#!/bin/bash

LIBS=`pkg-config --libs QtGui`

if [[ $? != 0 ]]; then
  echo "Qt4 paths could not be determined using pkg-config"
  echo "Aborting"
  exit
fi

QTPATH=`echo $LIBS | sed -e 's/ /\n/g' | grep ^-L | grep -i qt4 | cut -c 3-`

if [[ -z "$QTPATH" ]]; then
  echo "Qt4 library path could not be determined from the pkg-config output"
  echo "Aborting"
  exit
fi
  
if [[ ! -d "$QTPATH" ]]; then
  echo "Qt4 library path found ($QTPATH) is not a directory"
  echo "Aborting"
  exit
fi

if [[ ! -d "$QTPATH/plugins" ]]; then
  echo "Qt4 library path found ($QTPATH) does not contains a 'plugins' subdir"
  echo "Not sure what to do. Aborting"
  exit
fi

if [[ ! -d "$QTPATH/plugins/styles" ]]; then
  echo "Creating $QTPATH/plugins/styles"
  mkdir "$QTPATH/plugins/styles"
fi

echo "Coping style file in $QTPATH/plugins/styles"
cp libfloatstyle.so "$QTPATH/plugins/styles"

if [[ -z "$KDEDIR" ]]; then
  echo "KDEDIR variable not found"
  echo "KDE4 style not installed"
  exit
else
  echo "Is '$KDEDIR' the KDE4 path? (y/n)"
  read CONFIRMATION
  if [[ $CONFIRMATION = 'y' ]]; then
    echo "Coping KDE4 style description in $KDEDIR/share/apps/kstyle/themes"
    cp float.themerc "$KDEDIR/share/apps/kstyle/themes"
  fi
fi

