#!/bin/bash

LIBS=`pkg-config --libs QtGui`

if [[ $? != 0 ]]; then
  echo "Qt4 paths could not be determined using pkg-config"
  echo "Aborting"
  exit
fi

QTPATH=`echo $LIBS | sed -e 's/ /\n/g' | grep ^-L | grep -i qt4 | cut -c 3-`

if [[ -z "$QTPATH" ]]; then
  echo "Qt4 library path could not be determined from the pkg-config output"
  echo "Aborting"
  exit
fi
  
if [[ ! -d "$QTPATH" ]]; then
  echo "Qt4 library path found is not a directory"
  echo "Aborting"
  exit
fi

if [[ ! -d "$QTPATH/plugins" ]]; then
  echo "Qt4 library path found does not contains a 'plugins' subdir"
  echo "Not sure what to do. Aborting"
  exit
fi

rm "$QTPATH/plugins/styles/libfloatstyle.so"
rmdir "$QTPATH/plugins/styles"

if [[ -z "$KDEDIR" ]]; then
  echo "KDEDIR variable not found"
  echo "KDE4 style not uninstalled"
  exit
else
  echo "Is '$KDEDIR' the KDE4 path? (y/N)"
  read CONFIRMATION
  if [[ "$CONFIRMATION" = 'y' ]]; then
    rm "$KDEDIR/share/apps/kstyle/themes/float.themerc"
  fi
fi
